@@ -84,7 +84,7 @@ Z = c(0, 1, 1, 0, 0.5)
 
 test_that("QuandTree knn works even with npoints < k", {
 
-  nn = lidR:::C_knn(X,Y, c(0,2), c(1,2), 8, 1)
+  nn = lidR:::C_knn(X,Y, c(0,2), c(0.8,2), 8, 1)
 
   expect <- matrix(c(4,5,1,3,2,0,0,0,3,5,4,2,1,0,0,0), ncol = 8, byrow = T)
 
